ANNOTATIONS = 'annotations'
ANNOTATION_OBJECTS = 'annotationObjects'
ATTRIBUTES = 'attributes'
ATTRIBUTE_VALUES = 'attributeValues'
BBOX = 'bbox'
BOX_BOUNDARIES_AND_DIMENSIONS = 'boxBoundariesAndDimensions'
CATEGORIES = 'categories'
CREATED_AT = 'createdAt'
UPDATED_AT = 'updatedAt'
COLOR = 'color'
CONFIDENCE = 'confidence'
DATA = 'data'
DESCRIPTION = 'description'
H = 'h'
ID = 'id'
IS_NORMALIZED = 'isNormalized'
DEST_PROJECT_ID = 'destinationProjectId'
IMAGE = 'image'
IMAGES = 'images'
IMG_FILES = 'imgFiles'
IS_USER_ANNOTATED = 'isUserAnnotated'
KEY = 'key'
LABEL = 'label'
LABEL_COLOR = 'labelColor'
LABEL_REF = 'labelRef'
LABEL_TEXT = 'labelText'
LABELS = 'labels'
LINE = 'line'
METADATA = 'metadata'
META_UPDATES_ARRAY = 'metaUpdatesArray'
NAME = 'name'
OBJECT_KEY = 'objectKey'
POINTS = 'points'
POLYGON = 'polygon'
REF = 'ref'
SHAPE_ID = 'shapeId'
SUPERCATEGORY = 'supercategory'
TEXTS = 'texts'
TYPE = 'type'
USERNAME = 'userName'
VALUE_TEXT = 'valueText'
VALUES = 'values'
VALUE = 'value'
W = 'w'
X = 'x'
Y = 'y'
SESSION_ID = "sessionId"
TOTAL_IMAGE_COUNT = "totalImageCount"
UPLOADED_IMAGE_COUNT = "uploadedImageCount"
SELECTION_ID = "selectionId"
USER_ID = "userId"
COLLECTION_NAME = "collectionName"
IS_APPLY_TO_ALL_FILES = "isApplyToAllFiles"
FILES = "files"
OBJECT_TYPE = "objectType"
STORAGE_PATH = "storagePath"
BUCKET_NAME = "bucketName"
STORAGE_NAME = "storageName"